package objdbTest2;

import java.io.Serializable;

import javax.persistence.Basic;
import javax.persistence.Entity;
import javax.persistence.EntityManager;
import javax.persistence.FlushModeType;
import javax.persistence.Id;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.NoResultException;
import javax.persistence.TypedQuery;
import javax.persistence.Version;


@Entity
@NamedQueries({
	@NamedQuery(name = "objdbTest2.Singleton.getItem", query = "SELECT o FROM Item o WHERE o.cachedKey = :cachedKey"),
	@NamedQuery(name = "objdbTest2.Singleton.count", query = "SELECT COUNT(x) FROM Item x")
})
public class Singleton implements Serializable {

	public static Singleton getInstance(EntityManager em) {
		Singleton c = em.find(Singleton.class, "1");
		if (c == null) {
			c = new Singleton();
			em.persist(c);
		}
		return c;
	}

	@Id
	protected String id = "1";

	@SuppressWarnings("unused")
	// used by JPA
	@Version
	private int version;

	@Basic
	protected java.lang.Integer lastInstance;


	public void generateItems(EntityManager em, int howMany) {
		if (lastInstance == null) {
			lastInstance = 0;
		}
		int endCnt = lastInstance + howMany;
		int startCnt = lastInstance;
		while (endCnt > startCnt) {
			startCnt++;
			Item item = new Item(this, startCnt);
			em.persist(item);
			lastInstance = startCnt;
		}
	}
	
	public Item findItem(EntityManager em, int which) {
		TypedQuery<Item> q = em.createNamedQuery("objdbTest.Singleton.getItem", Item.class);
		q.setParameter("cachedKey", Integer.toString(which));
		try {
			return q.getSingleResult();
		} catch (NoResultException e) {
			return null;
		}
		
	}

	public long howManyItemsExist(EntityManager em) {
		TypedQuery<Long> q = em.createNamedQuery("objdbTest.Singleton.count", Long.class);
		q.setFlushMode(FlushModeType.AUTO);
		try {
			return q.getSingleResult();
		} catch (javax.persistence.NoResultException ex) {
			return -1;
		}
	}	

	private static final long serialVersionUID = -6107937486569751566L;
}
