package objdbTest;

import java.io.IOException;
import java.io.PrintWriter;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.transaction.HeuristicMixedException;
import javax.transaction.HeuristicRollbackException;
import javax.transaction.NotSupportedException;
import javax.transaction.RollbackException;
import javax.transaction.SystemException;


/**
 * Servlet implementation class TestServlet
 */
@WebServlet("/TestServlet")
public class TestServlet extends HttpServlet {
	private static final long serialVersionUID = 1L;
	
	
	private static final int AMOUNT = 1000;
       
    /**
     * @see HttpServlet#HttpServlet()
     */
    public TestServlet() {
        super();
    }

	/**
	 * @see HttpServlet#doGet(HttpServletRequest request, HttpServletResponse response)
	 */
	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		try {
			ResultStuff result = addItems(AMOUNT);
			if(-1 == result.newTotal) {
				outputError(response, "Did get new mmount");
			}
			else {
				outputTotal(response, result);
			}
		} catch (ExampleException e) {
			outputError(response, e.getMessage());
		}
	}
	
	

	private void outputTotal(HttpServletResponse response, ResultStuff result) {
		try {
			PrintWriter out = response.getWriter();
			out.println(
					"<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">" +
					"<html><head>" +
					"<meta http-equiv=\"Content-Type\" content=\"text/html; charset=ISO-8859-1\">"+
					"<title>Result</title>" +
					"</head>" +   
					"<body>" + 
						"<h3>Added " + AMOUNT + " and got " + result.newTotal + "</h3>" +
						"<h3>Found " + (result.item==null?"null":result.item.getInfo()) + "</h3>" +
						"<a href=\"TestServlet\">Add another " + AMOUNT + "</a>" +
					"</body>" +
					"</html>");
			out.flush();
		} catch (IOException e) {
			try {
				response.sendError(500, "Cannot be print writer");
			} catch (IOException e1) {
				e1.printStackTrace();
			}
		}
		
	}

	private void outputError(HttpServletResponse response, String message) {try {
		PrintWriter out = response.getWriter();
		out.println(
				"<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">" +
				"<html><head>" +
				"<meta http-equiv=\"Content-Type\" content=\"text/html; charset=ISO-8859-1\">"+
				"<title>Result</title>" +
				"</head>" +   
				"<body>" + 
					"<h3>Oh No Something went wrong - look at the logs</h3>" +
					"<br/>"+ message + "<br/>" +
					"<a href=\"TestServlet\">Try again</a>" +
				"</body>" +
				"</html>");
		out.flush();
	} catch (IOException e) {
		try {
			response.sendError(500, "Cannot be print writer");
		} catch (IOException e1) {
			e1.printStackTrace();
		}
	}
	}

	private static class ResultStuff {
		long newTotal = -1;
		Item item = null;
		Item item2 = null;
		
	}
	
	private static class ExampleException extends Exception {

		public ExampleException(String message) {
			super(message);
		}
		
	}
	private ResultStuff addItems(int howMany) throws ExampleException {
		ResultStuff results = new ResultStuff();
		try {
			InitialContext context = new InitialContext();
			javax.transaction.UserTransaction tx = (javax.transaction.UserTransaction) context
					.lookup("java:comp/UserTransaction");
			tx.begin();
			SLSBObjDbRemote bean = (SLSBObjDbRemote) context
					.lookup("SLSBObjDb");
			long oldAmount = bean.countItems();
			if(oldAmount + howMany > Integer.MAX_VALUE) {
				System.err.println("Can't add any more");
				throw new ExampleException("Can't have any more than MaxInt items");
			}
			bean.addItems(howMany);
			double which = oldAmount + (((double)howMany) / 2);
			results.item = bean.findItem((int)which);
			results.newTotal = bean.countItems();
			if(results.newTotal < (howMany+oldAmount)) {
				System.err.println("Only found " + results.newTotal + " items expected " + (howMany + oldAmount));
			}
			else {
				System.err.println("Found " + results.newTotal + " items as expected = " + (howMany + oldAmount));
			}
			tx.commit();
		} catch (NamingException e) {
			e.printStackTrace();
		} catch (NotSupportedException e) {
			e.printStackTrace();
		} catch (SystemException e) {
			e.printStackTrace();
		} catch (SecurityException e) {
			e.printStackTrace();
		} catch (IllegalStateException e) {
			e.printStackTrace();
		} catch (RollbackException e) {
			e.printStackTrace();
		} catch (HeuristicMixedException e) {
			e.printStackTrace();
		} catch (HeuristicRollbackException e) {
			e.printStackTrace();
		} finally {
		}
		return results;
	}

}
